"""hcai_audioset dataset."""

import tensorflow_datasets as tfds
import tensorflow as tf
import pathlib
import json
import os
import pickle

# TODO(hcai_audioset): Markdown description  that will appear on the catalog page.
_DESCRIPTION = """
Description is **formatted** as markdown.

It should also contain any processing which has been applied (if any),
(e.g. corrupted example skipped, images cropped,...):
"""

# TODO(hcai_audioset): BibTeX citation
_CITATION = """
"""


class HcaiAudioset(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for hcai_audioset dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    with open(os.path.join( os.path.dirname(__file__),'ontology.json')) as o:
        ontology = json.load(o)

    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            # These are the features of your dataset like images, labels ...
            'audio': tfds.features.Text(),
            'label': tfds.features.ClassLabel(names=[x['id'] for x in ontology])
        }),
        # If there's a common (input, target) tuple from the
        # features, specify them here. They'll be used if
        # `as_supervised=True` in `builder.as_dataset`.
        supervised_keys=('audio', 'label'),  # Set to `None` to disable
        homepage='https://dataset-homepage/',
        citation=_CITATION,
        metadata=tfds.core.MetadataDict(ontology=ontology)
    )

  def _populate_metadata(self, directory):
    m_path = directory / 'raw' / 'mapping.pickle'
    with open(m_path, 'rb') as m:
        self._mapping = {k : list(v) for k,v in pickle.load(m).items()}
    print('')

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    extracted_dir = pathlib.Path(r'Z:\AudioSet')
    self._populate_metadata(extracted_dir)
    splits = ['train']
    return { s: self._generate_examples(extracted_dir) for s in splits }

  def _generate_examples(self, path):
    """Yields examples."""
    for label, samples in self._mapping.items():
        for s in list(samples):
            yield s + '_' + label, {
              'audio': os.path.join(path, 'raw', 'files', s),
              'label': label,
            }



"""hcai_ckplus dataset."""

import os
import glob
import tensorflow_datasets as tfds
import random

# TODO(hcai_ckplus): Markdown description  that will appear on the catalog page.
_DESCRIPTION = """
Description is **formatted** as markdown.

It should also contain any processing which has been applied (if any),
(e.g. corrupted example skipped, images cropped,...):
"""

# TODO(hcai_ckplus): BibTeX citation
_CITATION = """
"""


class HcaiCkplus(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for hcai_ckplus dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
    '1.0.0': 'Initial release.',
  }

  def __init__(self, *, dataset_dir=None, **kwargs):
    self.dataset_dir = dataset_dir
    self.labels = ['neutral', 'anger', 'contempt', 'disgust', 'fear', 'happy', 'sadness', 'suprise']
    super(HcaiCkplus, self).__init__(**kwargs)

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    # TODO(hcai_ckplus): Specifies the tfds.core.DatasetInfo object
    return tfds.core.DatasetInfo(
      builder=self,
      description=_DESCRIPTION,
      features=tfds.features.FeaturesDict({
        # These are the features of your dataset like images, labels ...
        'image': tfds.features.Image(shape=(None, None, 3)),
        # 0=neutral, 1=anger, 2=contempt, 3=disgust, 4=fear, 5=happy, 6=sadness, 7=surprise)
        'label': tfds.features.ClassLabel(names=self.labels),
      }),
      # If there's a common (input, target) tuple from the
      # features, specify them here. They'll be used if
      # `as_supervised=True` in `builder.as_dataset`.
      supervised_keys=('image', 'label'),  # Set to `None` to disable
      homepage='https://dataset-homepage/',
      citation=_CITATION,
    )

  def _standard_splits(self):
    """Returns the standard splits for the dataset.
    Since clplus has no predefined splits the function applies a 70-15-15 split ratio to create them.
    Splits are sorted by filename and contain only unique subjects."""

    emotion_files = glob.glob(os.path.join(self.dataset_dir, 'Emotion', '**/**/*.txt'))
    samples = []

    for ef in emotion_files:
      with open(ef, 'r') as f:
        emotion = self.labels[int(float(f.readline().strip()))]
        fn = ef.split(os.sep)[-1].replace('_emotion.txt', '.png')
        samples.append((fn, emotion))

    # since we do not have any specific neutral images directly labeled we use the first image of every sequence
    samples.extend([(x.split(os.sep)[-1], 'neutral') for x in sorted(glob.glob(os.path.join(self.dataset_dir, 'cohn-kanade-images', '**/**/*01.png')))])

    unique_subjects = list(set([x[0].split('_')[0] for x in samples]))
    random.Random(1337).shuffle(unique_subjects)

    train = [s for u in unique_subjects[:int(len(unique_subjects) * 0.7)] for s in samples if s[0].startswith(u)]
    val = [s for u in unique_subjects[int(len(unique_subjects) * 0.7):int(len(unique_subjects) * 0.85)] for s in samples if s[0].startswith(u)]
    test = [s for u in unique_subjects[int(len(unique_subjects) * 0.85):] for s in samples if s[0].startswith(u)]

    return train, val, test

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""

    train, val, test = self._standard_splits()
    return {
      'train': self._generate_examples(train),
      'val': self._generate_examples(val),
      'test': self._generate_examples(test),
    }

  def _generate_examples(self, files):
    """Yields examples."""

    for f, e in files:
      yield f, {
        'image': os.path.join(self.dataset_dir, 'cohn-kanade-images', *f.split('_')[:-1], f),
        'label': e,
      }



"""hcai_librispeech dataset."""

import tensorflow_datasets as tfds
import tensorflow as tf
import pathlib
import os

_DESCRIPTION = """
LibriSpeech is a corpus of approximately 1000 hours of read English speech with sampling rate of 16 kHz,
prepared by Vassil Panayotov with the assistance of Daniel Povey. The data is derived from read
audiobooks from the LibriVox project, and has been carefully segmented and aligned.87
"""

_CITATION = """
  title={Librispeech: an ASR corpus based on public domain audio books},
  author={Panayotov, Vassil and Chen, Guoguo and Povey, Daniel and Khudanpur, Sanjeev},
  booktitle={Acoustics, Speech and Signal Processing (ICASSP), 2015 IEEE International Conference on},
  pages={5206--5210},
  year={2015},
  organization={IEEE}
}"""

class HcaiLibrispeech(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for hcai_librispeech dataset."""

    VERSION = tfds.core.Version('1.0.0')
    RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
    }

    def __init__(self, *, dataset_dir=None, **kwargs):
        super(HcaiLibrispeech, self).__init__(**kwargs)
        self.dataset_dir = os.path.join(dataset_dir)

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""
        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict({
                "speech": tfds.features.Text(),
                "text": tfds.features.Text(),
                "speaker_id": tf.int64,
                "chapter_id": tf.int64,
                "id": tf.string,
            }),
            # If there's a common (input, target) tuple from the
            # features, specify them here. They'll be used if
            # `as_supervised=True` in `builder.as_dataset`.
            supervised_keys=('speech', 'text'),  # Set to `None` to disable
            homepage='https://dataset-homepage/',
            citation=_CITATION,
            metadata=tfds.core.MetadataDict(sample_rate=16000,),
        )

    def _populate_metadata(self, directory):
        # All dirs contain the same metadata.
        self.info.metadata["speakers"] = self._read_metadata_file(
          os.path.join(directory, "LibriSpeech/SPEAKERS.TXT"),
          ["speaker_id", "gender", "subset", "minutes", "name"])
        self.info.metadata["chapters"] = self._read_metadata_file(
          os.path.join(directory, "LibriSpeech/CHAPTERS.TXT"), [
              "chapter_id", "speaker_id", "minutes", "subset", "project_id",
              "book_id", "chapter_title", "project_title"
          ])

    def _read_metadata_file(self, path, field_names):
        metadata = {}
        with tf.io.gfile.GFile(path) as f:
            for line in f:
                if line.startswith(";"):
                    continue
                fields = line.split("|", len(field_names))
                metadata[int(fields[0])] = {
                    k: v.strip() for k, v in zip(field_names[1:], fields[1:])
                }
        return metadata

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        self._populate_metadata()

        splits = ['dev-clean', 'dev-other', 'test-clean', 'test-other', 'train-clean-100', 'train-clean-360', 'train-other-500']
        return { s: self._generate_examples(self.dataset_dir / 'Librispeech' / s) for s in splits }

    def _generate_examples(self, directory):
        transcripts_glob = os.path.join(directory, "*/*/*.txt")
        for transcript_file in tf.io.gfile.glob(transcripts_glob):
            path = os.path.dirname(transcript_file)
            with tf.io.gfile.GFile(os.path.join(path, transcript_file)) as f:
                for line in f:
                    line = line.strip()
                    key, transcript = line.split(" ", 1)
                    audio_file = "%s.flac" % key
                    speaker_id, chapter_id = [int(el) for el in key.split("-")[:2]]
                    example = {
                      "id": key,
                      "speaker_id": speaker_id,
                      "chapter_id": chapter_id,
                      "speech": os.path.join(path, audio_file),
                      "text": transcript
                    }
                    yield key, example


"""hcai_nova_dynamic dataset."""

import tensorflow_datasets as tfds
import os
import cv2
import sys
import numpy as np
from hcai_datasets.hcai_nova_dynamic.nova_db_handler import NovaDBHandler
from hcai_datasets.hcai_nova_dynamic import data_utils
from hcai_datasets.hcai_nova_dynamic.defines import DataTypes

# TODO(hcai_audioset): Markdown description  that will appear on the catalog page.
_DESCRIPTION = """
The Nova Dynamic dataset can be used to retrieve the data and labels for a certain session or a certain part of a session of a nova dataset. 
This is part of the Nova CML Python backend (https://github.com/hcmlab/nova)
To specify which data to load use the following format: 

TODO: x
 
"""

# TODO(hcai_audioset): BibTeX citation
_CITATION = """
"""


class HcaiNovaDynamic(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for hcai_nova_dynamic dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
    '1.0.0': 'Initial release.',
  }

  def __init__(self, *, db_config_path=None, db_config_dict=None, dataset=None, nova_data_dir=None, sessions=None, annotator=None,
               schemes=None, roles=None, data_streams=None, start=0, end=sys.float_info.max, left_context, right_context, frame_step, **kwargs):
    """
    Initialize the HcaiNovaDynamic dataset builder
    Args:
      db_config_path: path to a configfile whith the nova database config.
      db_config_dict: dictionary with the nova database config. can be used instead of db_config_path. if both are specified db_config_dict is used.
      dataset: the name of the dataset. must match the dataset name in the nova database.
      sessions: list of sessions that should be loaded. must match the session names in nova.
      annotator: the name of the annotator that labeld the session. must match annotator names in nova.
      schemes: list of the annotation schemes to fetch
      roles: list of roles for which the annotation should be loaded.
      data_streams: list datastreams for which the annotation should be loaded. must match stream names in nova.
      start: optional start time_ms. use if only a specific chunk of a session should be retreived.
      end: optional end time_ms. use if only a specifc chunk of a session should be retreived.
      **kwargs: arguments that will be passed through to the dataset builder
    """
    self.dataset = dataset
    self.nova_data_dir = nova_data_dir
    self.sessions = sessions
    self.roles = roles
    self.annotator = annotator
    self.left_context = left_context
    self.right_context = right_context
    self.frame_step = frame_step
    self.start = start
    self.end = end
    self.nova_db_handler = NovaDBHandler(db_config_path, db_config_dict)

    mongo_schemes = self.nova_db_handler.get_schemes(dataset=dataset, schemes=schemes)
    mongo_streams = self.nova_db_handler.get_data_streams(dataset=dataset, data_streams=data_streams)

    # infos as needed for the tensorflow dataset init and     # additional info for loading samples
    self._info_label = self._get_label_info_from_mongo_doc(mongo_schemes)
    self._info_data = self._get_data_info_from_mongo_doc(mongo_streams)

    super(HcaiNovaDynamic, self).__init__(**kwargs)

  def _info(self) -> tfds.core.DatasetInfo:
      """Returns the dataset metadata."""

      return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({**self._info_label, **{k:v['feature'] for k,v in self._info_data.items() }}),
        # If there's a common (input, target) tuple from the
        # features, specify them here. They'll be used if
        # `as_supervised=True` in `builder.as_dataset`.
        supervised_keys=(None),  # Set to `None` to disable
        homepage='https://github.com/hcmlab/nova',
        citation=_CITATION
      )

  def _get_label_info_from_mongo_doc(self, mongo_schemes):
    label_info = {}
    # List of all combinations from roles and schemes that occur in the retreived data. Form is 'role.scheme'
    #roles_with_scheme = set([rs for session in self.annos.values() for rs in list(session.keys())])
    for scheme in mongo_schemes:
        # List of all role where the current scheme occures in the retrieved data.
        # label_keys = list(filter(lambda x : scheme['name'] in x,  roles_with_scheme))
        for role in self.roles:
          if scheme['type'] == 'DISCRETE':
            label_info[role + '.' + scheme['name']] = tfds.features.ClassLabel(names=[x['name'] for x in sorted(scheme['labels'], key=lambda k: k['id'])])
          else:
            raise ValueError('Invalid label type {}'.format(scheme['type']))
    return label_info

  def _get_video_resolution(self, path):
    vcap = cv2.VideoCapture(path)
    # get vcap property
    width  = int(vcap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(vcap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    vcap.release()
    return (height, width)

  def _get_data_info_from_mongo_doc(self, mongo_data_streams):
      data_info = {}
      for data in mongo_data_streams:
        for role in self.roles:
          if data['type'] == 'video':
            # We can use any role for testing since Nova assumes that there will be a stream for every role
            role = self.roles[0]
            sample_stream_name = role + '.' + data['name'] + '.' + data['fileExt']
            sample_stream_path = os.path.join(self.nova_data_dir, self.dataset, self.sessions[0], sample_stream_name)
            res = self._get_video_resolution(sample_stream_path)
            # shape is (None, H, W, C) - We assume that we always have three channels
            data_shape = (None, ) + res + (3, )
            data_id = role + '.' + data['name']
            data_info[data_id] = {'feature' : tfds.features.Video(data_shape), 'file': sample_stream_name, 'sr': data['sr'], 'type': DataTypes.VIDEO}
          elif data['type'] == 'audio':
            pass
          else:
            raise ValueError('Invalid data type {}'.format(data['type']))

      return data_info

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    return {'dynamic_split': self._generate_examples()}

  def _get_label_for_frame(self, annotation, start, end):

    if annotation == -1:
      return -1

    else:
      annos_for_sample = list(filter(lambda x: x['from'] >= start and x['to'] <= end, annotation))

      if not annos_for_sample:
        return -1

      majority_sample_idx = np.argmax(list(map(lambda x: x['to'] - x['from'], annos_for_sample)))

      return annos_for_sample[majority_sample_idx]['id']

  def _get_data_for_frame(self, file_reader, feature_type, sr, start, end):
    start_frame = data_utils.time_to_frame(sr, start)
    end_frame = data_utils.time_to_frame(sr, end)
    if feature_type == DataTypes.VIDEO:
      return data_utils.chunk_vid(file_reader, start_frame, end_frame)

  def _generate_examples(self):
    """Yields examples."""
    # Fetching all annotations that are available for the respective schemes and roles
    for session in self.sessions:

      annotation = {}
      data = {}

      # loading annotations for the session
      for l in self._info_label.keys():
        r, s = l.split('.')
        annotation[l] = self.nova_db_handler.get_annos(self.dataset, s, session, self.annotator, r)

      # openening data reader for this session
      for d, feature_info in self._info_data.items():
        data_path = os.path.join(self.nova_data_dir, self.dataset, session, feature_info['file'])

        if not os.path.isfile(data_path):
          raise FileNotFoundError('No datastream found at {}'.format(data_path))
        file_reader = data_utils.open_file_reader(data_path, feature_info['type'])
        data[d] = file_reader

      session_info = self.nova_db_handler.get_session_info(self.dataset, session)[0]
      dur = session_info['duration']

      # current position of the last frame in seconds
      c_pos = 0

      # increase current position till we reach the first frame
      while c_pos - (self.left_context + self.frame_step) < self.start:
        c_pos += self.frame_step

      # generate samples for this session
      while c_pos + self.right_context < min(self.end, dur):
        sample_start = c_pos - (self.left_context + self.frame_step)
        sample_end = c_pos + self.right_context
        key = str(sample_start) + '_' + str(sample_end)

        labels_for_sample = [{ k: self._get_label_for_frame(v, sample_start, sample_end)} for k, v in annotation.items()]
        data_for_sample = [{ k: self._get_data_for_frame(v, self._info_data[k]['type'], self._info_data[k]['sr'], sample_start, sample_end)} for k, v in data.items()]

        #sample_dict = {
        #  'video' : np.ones( (20,288,180,3), dtype=np.uint8)
        #}
        sample_dict = {}
        for l in labels_for_sample:
          sample_dict.update(l)

        for d in data_for_sample:
          sample_dict.update(d)

        yield key, sample_dict
        c_pos += self.frame_step

      # closing file readers for this session
      for d, fr in data.items():
        data_utils.close_file_reader(fr, self._info_data[d]['feature'])


'''Image Datasets'''
from hcai_datasets import hcai_ckplus
from hcai_datasets import hcai_faces

'''Audio Datasets'''
from hcai_datasets import hcai_audioset
from hcai_datasets import hcai_librispeech
from hcai_datasets import hcai_is2021_ess

'''Nova Datasets'''
from hcai_datasets import hcai_nova_dynamic



https://www.tensorflow.org/tutorials/audio/transfer_learning_audio
# Util functions for loading audio files and ensure the correct sample rate

@tf.function
def load_wav_16k_mono(filename):
  """ read in a waveform file and convert to 16 kHz mono """
  file_contents = tf.io.read_file(filename)
  wav, sample_rate = tf.audio.decode_wav(
    file_contents,
    desired_channels=1)
  wav = tf.squeeze(wav, axis=-1)
  sample_rate = tf.cast(sample_rate, dtype=tf.int64)
  wav = tfio.audio.resample(wav, rate_in=sample_rate, rate_out=16000)
  return wav

testing_wav_data = load_wav_16k_mono(testing_wav_file_name)

_ = plt.plot(testing_wav_data)

# Play the audio file.
display.Audio(testing_wav_data,rate=16000)


"""hcai_faces dataset."""

import os

import tensorflow as tf
import tensorflow_datasets as tfds

# TODO(hcai_faces): Markdown description  that will appear on the catalog page.
_DESCRIPTION = """
Description is **formatted** as markdown.

It should also contain any processing which has been applied (if any),
(e.g. corrupted example skipped, images cropped,...):
"""

# TODO(hcai_faces): BibTeX citation
_CITATION = """
"""


class HcaiFaces(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for hcai_faces dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
    '1.0.0': 'Initial release.',
  }

  def __init__(self, *, dataset_dir=None, **kwargs):
    super(HcaiFaces, self).__init__(**kwargs)
    self.dataset_dir = os.path.join(dataset_dir, 'bilder')

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return tfds.core.DatasetInfo(
      builder=self,
      description=_DESCRIPTION,
      features=tfds.features.FeaturesDict({
        # These are the features of your dataset like images, labels ...
        'image': tfds.features.Image(shape=(None, None, 3)),
        'id': tf.int64,
        'age': tfds.features.ClassLabel(names=['y', 'o', 'm']),
        'gender': tfds.features.ClassLabel(names=['m', 'f']),
        'emotion': tfds.features.ClassLabel(names=['a', 'd', 'f', 'h', 'n', 's']),
        'set': tfds.features.ClassLabel(names=['a', 'b']),
      }),
      # If there's a common (input, target) tuple from the
      # features, specify them here. They'll be used if
      # `as_supervised=True` in `builder.as_dataset`.
      supervised_keys=('image', 'emotion'),  # Set to `None` to disable
      homepage='https://dataset-homepage/',
      citation=_CITATION,
    )

  def _standard_splits(self):
    """Returns the standard splits for the dataset.
    Since faces has no predefined splits the function applies a 70-15-15 split ratio to create them.
    Splits are sorted by filename and contain unique subjects and equal gender distribution."""

    f_names = os.listdir(self.dataset_dir)
    f_names_unique = []

    for f in f_names:
      if f.endswith('jpg'):
        f_names_unique.append((f.split('_')[1], f.split('_')[0]))

    f_names_unique = set(f_names_unique)

    # sorting unique subject ids into age categories
    old = sorted([x[1] for x in f_names_unique if x[0] == 'o'])
    middle = sorted([x[1] for x in f_names_unique if x[0] == 'm'])
    young = sorted([x[1] for x in f_names_unique if x[0] == 'y'])

    # building splits using 70,15,15 % distribution evenly over all age groups
    f = 0
    t = 0.7
    train_ids = old[int(len(old) * f): int(len(old) * t)] + \
                middle[int(len(middle) * f): int(len(middle) * t)] + \
                young[int(len(young) * f): int(len(young) * t)]
    f = 0.7
    t = 0.85
    val_ids = old[int(len(old) * f): int(len(old) * t)] + \
              middle[int(len(middle) * f): int(len(middle) * t)] + \
              young[int(len(young) * f): int(len(young) * t)]
    f = 0.85
    t = 1
    test_ids = old[int(len(old) * f): int(len(old) * t)] + \
               middle[int(len(middle) * f): int(len(middle) * t)] + \
               young[int(len(young) * f): int(len(young) * t)]

    # getting filenames for each split
    train = [f for x in train_ids for f in f_names if f.startswith(x)]
    val = [f for x in val_ids for f in f_names if f.startswith(x)]
    test = [f for x in test_ids for f in f_names if f.startswith(x)]

    return train, val, test

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    train, val, test = self._standard_splits()
    return {
      'train': self._generate_examples(train),
      'val': self._generate_examples(val),
      'test': self._generate_examples(test),
    }

  def _generate_examples(self, files):
    """Yields examples."""
    for f in files:
      # names are composed as the following:
      # "ID_ageGroup_gender_emotion_pictureSet.jpg
      id, age, gender, emotion, set = os.path.splitext(f)[0].split('_')

      yield f, {
        'image': os.path.join(self.dataset_dir, f),
        'id': int(id),
        'age': age,
        'gender': gender,
        'emotion': emotion,
        'set': set
      }



import configparser
from pymongo import MongoClient

ANNOTATOR_COLLECTION = 'Annotators'
SCHEME_COLLECTION = 'Schemes'
STREAM_COLLECTION = 'Streams'
ROLE_COLLECTION = 'Roles'
ANNOTATION_COLLECTION = 'Annotations'
SESSION_COLLECTION = 'Sessions'
ANNOTATION_DATA_COLLECTION = 'AnnotationData'

class NovaDBHandler():

  def __init__(self, db_config_path=None, db_config_dict=None):

    # Connecting to the database
    if db_config_path:
      cfg = self.read_config(db_config_path)
      self.ip = str(cfg['DB']['ip'])
      self.port = int(cfg['DB']['port'])
      self.user = str(cfg['DB']['user'])
      self.password = str(cfg['DB']['password'])

    # If a db config_dict is specified overwrite the config from path
    if db_config_dict:
      if db_config_path:
        print('WARNING! database config are specifed as file AND and as dictionary. Using the dictionary.')
      self.ip = db_config_dict['ip']
      self.port = db_config_dict['port']
      self.user = db_config_dict['user']
      self.password = db_config_dict['password']

    if not (self.ip or self.port or self.user or self.password):
      print('WARNING! No valid nova database config found for path {} and dict {} \n Found config parameters are ip:{}, port{}, user: {}. Also check your password.'.format(db_config_path, db_config_dict, self.ip, self.port, self.user))

    self.client = MongoClient(host=self.ip, port=self.port, username=self.user, password=self.password)
    self.datasets = self.client.list_database_names()

  def print_config(self, cfg, cfg_path):
    print('Loaded config from {}:'.format(cfg_path))
    print('---------------------')
    for sec_name, sec_dict in cfg._sections.items():
      print(sec_name)
      for k, v in sec_dict.items():
        if k == 'password':
          continue
        else:
          print('\t{} : {}'.format(k, v))
    print('---------------------')

  def read_config(self, cfg_path):
    config = configparser.RawConfigParser()
    config.read(cfg_path)
    self.print_config(config, cfg_path)
    return config

  def get_docs_by_prop(self, vals, property, database, collection):
    """

    Args:
      vals:
      property:
      database:
      collection:
      client:

    Returns:

    """
    filter = []

    if not type(vals) == type(list()):
      vals = [vals]

    for n in vals:
      filter.append({property: n})

    filter = {"$or": filter}
    ret = list(self.client[database][collection].find(filter))
    return ret

  def get_schemes(self, dataset, schemes):
    """
    Fetches the scheme object that matches the specified criteria from the nova database and returns them as a python readable dictionary.
    Args:
      ip:
      port:
      user:
      password:
      dataset:
      scheme:

    Returns:

    """
    if not dataset in self.datasets:
      raise ValueError('{} not found in datasets'.format(dataset))

    mongo_schemes = []
    for scheme in schemes:
      mongo_schemes.append(self.get_docs_by_prop(scheme, 'name', dataset, SCHEME_COLLECTION)[0])

    if not mongo_schemes:
      raise ValueError('no entries with scheme {} found'.format(schemes))

    return mongo_schemes

  def get_session_info(self, dataset, session):
    """
    Fetches the session object that matches the specified criteria from the nova database and returns them as a python readable dictionary.

    Args:
      dataset:
      session:

    Returns:

    """
    mongo_session = self.get_docs_by_prop(session, 'name', dataset, SESSION_COLLECTION)
    return mongo_session


  def get_data_streams(self, dataset, data_streams):
    """
    Fetches the stream objects that matches the specified criteria from the nova database and returns them as a python readable dictionary.
    Args:
      dataset:
      session:
      role_list:
      data_stream_list:
    """
    if not dataset in self.datasets:
      raise ValueError('{} not found in datasets'.format(dataset))

    mongo_streams = []
    for stream in data_streams:
      mongo_streams.append(self.get_docs_by_prop(stream, 'name', dataset, STREAM_COLLECTION)[0])

    if not mongo_streams:
      raise ValueError('no entries with scheme {} found'.format(data_streams))

    return mongo_streams

  def get_annotation_docs(self, mongo_schemes, mongo_sessions, mongo_annotators, mongo_roles, database, collection):
    """
    Fetches all annotationobjects that match the specified criteria from the nova database
    Args:
      mongo_schemes:
      mongo_sessions:
      mongo_annotators:
      mongo_roles:
      database:
      collection:
      client:

    Returns:

    """
    scheme_filter = []
    role_filter = []
    annotator_filter = []
    session_filter = []

    for ms in mongo_schemes:
      scheme_filter.append({'scheme_id': ms['_id']})

    for mse in mongo_sessions:
      session_filter.append({'session_id': mse['_id']})

    for ma in mongo_annotators:
      annotator_filter.append({'annotator_id': ma['_id']})

    for mr in mongo_roles:
      role_filter.append({'role_id': mr['_id']})

    filter = {
      '$and': [
        {'$or': scheme_filter},
        {'$or': session_filter},
        {'$or': role_filter},
        {'$or': annotator_filter},
      ]
    }

    ret = list(self.client[database][collection].find(filter))
    return ret

  def get_annos(self, dataset, scheme, session, annotator, roles):
    """
    Fetches all annotations that matche the specified criteria from the nova database and returns them as a python readable dictionary.
    Args:
      ip:
      port:
      user:
      password:
      dataset:
      scheme:
      session:
      annotator:
      roles:

    Returns:

    """
    if not dataset in self.datasets:
      print('{} not found in datasets'.format(dataset))

    mongo_schemes = self.get_docs_by_prop(scheme, 'name', dataset, SCHEME_COLLECTION)
    if not mongo_schemes:
      print('no entries with scheme {} found'.format(scheme))
      exit()
    mongo_annotators = self.get_docs_by_prop(annotator, 'name', dataset, ANNOTATOR_COLLECTION)
    if not mongo_annotators:
      print('no entries for annotator {} found'.format(annotator))
      exit()
    mongo_roles = self.get_docs_by_prop(roles, 'name', dataset, ROLE_COLLECTION)
    if not mongo_roles:
      print('no entries for role {} found'.format(roles))
      exit()
    mongo_sessions = self.get_docs_by_prop(session, 'name', dataset, SESSION_COLLECTION)
    if not mongo_sessions:
      print('no entries for session {} found'.format(session))
      exit()

    mongo_annos = self.get_annotation_docs(mongo_schemes, mongo_sessions, mongo_annotators, mongo_roles, dataset, ANNOTATION_COLLECTION)

    # getting the annotation data and the session name
    if not mongo_annos:
      return -1

    else:
      label = self.get_docs_by_prop(mongo_annos[0]['data_id'], '_id', dataset, ANNOTATION_DATA_COLLECTION)
      label = label[0]['labels']

    return label


if __name__ == '__main__':

  db_handler = NovaDBHandler('db.cfg')

  dataset = 'DFG_A1_A2b'
  session = 'NP001'
  scheme = 'IEng'
  annotator = 'gold'
  roles = ['caretaker', 'infant']

  mongo_scheme = db_handler.get_schemes(dataset=dataset, scheme=scheme)
  mongo_annos = db_handler.get_annos(dataset=dataset, scheme=scheme, session=session, annotator=annotator, roles=roles)
  print('Done')



"""hcai_is2021_ess dataset."""

import tensorflow_datasets as tfds
import tensorflow as tf
import pathlib
import os

# TODO(hcai_is2021_ess): Markdown description  that will appear on the catalog page.
_DESCRIPTION = """
Description is **formatted** as markdown.

It should also contain any processing which has been applied (if any),
(e.g. corrupted example skipped, images cropped,...):
"""

# TODO(hcai_is2021_ess): BibTeX citation
_CITATION = """
"""


class HcaiIs2021Ess(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for hcai_is2021_ess dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    # TODO(hcai_is2021_ess): Specifies the tfds.core.DatasetInfo object
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            "speech": tfds.features.Text(),
            "label": tf.string,
            "id": tf.string,
            "text":  tfds.features.Text(),
        }),
        # If there's a common (input, target) tuple from the
        # features, specify them here. They'll be used if
        # `as_supervised=True` in `builder.as_dataset`.
        supervised_keys=('speech', 'label'),  # Set to `None` to disable
        homepage='https://dataset-homepage/',
        citation=_CITATION,
        metadata=tfds.core.MetadataDict(sample_rate=16000,)
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
      """Returns SplitGenerators."""
      label_dir = pathlib.Path(r'Z:\Interspeech\2021\ComParE2021_ESS\dist')
      splits = ['train', 'devel', 'test']
      return { s: self._generate_examples(os.path.join(label_dir), s) for s in splits }

  def _generate_examples(self, dist_dir, set):
      header = True
      with tf.io.gfile.GFile(os.path.join(dist_dir, 'lab', set + '.csv')) as f:
          for line in f:
              if header:
                  header = False
                  continue
              line = line.strip()
              fn, label = line.split(",", 1)
              fn = fn.split('.')[0]
              audio_file = "%s.wav" % fn
              transcript_file = "%s.txt" % fn
              transcript = ''
              transcript_clean = ''
              with tf.io.gfile.GFile(os.path.join(dist_dir, 'text', transcript_file)) as t:
                  for tl in t:
                        transcript = tl

              example = {
                  "id": fn,
                  "speech": os.path.join(dist_dir, 'wav', audio_file),
                  "text": transcript,
                  'label': label
              }
              yield fn, example




import cv2
import numpy as np
import tensorflow_datasets as tfds
from hcai_datasets.hcai_nova_dynamic.defines import DataTypes

def frame_to_time(sr: int, frame: int):
  return frame / sr

def time_to_frame(sr: int, time_s: float):

  # Last frame will not be included if the product is not an int!
  return int(time_s * sr)

def chunk_vid(vcap: cv2.VideoCapture, start_frame: int, end_frame: int):
  vcap.set(cv2.CAP_PROP_POS_FRAMES, start_frame)

  width = int(vcap.get(cv2.CAP_PROP_FRAME_WIDTH))
  heigth = int(vcap.get(cv2.CAP_PROP_FRAME_HEIGHT))
  depth = 3
  length = end_frame - start_frame

  chunk = np.zeros( (length, heigth, width, depth), dtype=np.uint8)

  for i in range(length):
    ret, frame = vcap.read()

    if not ret:
      raise IndexError('Video frame {} out of range'.format(i))

    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
    chunk[i] = frame

  return chunk

def open_file_reader(path, feature_type):
  if feature_type == DataTypes.VIDEO:
    return cv2.VideoCapture(path)

def close_file_reader(reader, feature_type):
  if feature_type == DataTypes.VIDEO:
    return reader.release()
